@@ -2286,15 +2286,6 @@ MGAPreInit(ScrnInfoPtr pScrn, int flags)
        }
     }
 #endif
-    pMga->CurrentLayout.bitsPerPixel = pScrn->bitsPerPixel;
-    pMga->CurrentLayout.depth = pScrn->depth;
-    pMga->CurrentLayout.displayWidth = pScrn->displayWidth;
-    pMga->CurrentLayout.weight.red = pScrn->weight.red;
-    pMga->CurrentLayout.weight.green = pScrn->weight.green;
-    pMga->CurrentLayout.weight.blue = pScrn->weight.blue;
-    pMga->CurrentLayout.Overlay8Plus24 = pMga->Overlay8Plus24;
-    pMga->CurrentLayout.mode = pScrn->currentMode;
-	
 
     xf86SetPrimInitDone(pScrn->entityList[0]);
     
@@ -2822,11 +2813,6 @@ MGAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     pMga->BlockHandler = pScreen->BlockHandler;
     pScreen->BlockHandler = MGABlockHandler;
 
-#if 0
-    if(!pMga->ShadowFB) /* hardware cursor needs to wrap this layer */
-	MGADGAInit(pScreen);
-#endif
-
     if (!pMga->NoAccel) {
 #ifdef USE_EXA
 	if (pMga->Exa)
@@ -2838,15 +2824,6 @@ MGAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 #endif
     }
 
-    pMga->CurrentLayout.bitsPerPixel = pScrn->bitsPerPixel;
-    pMga->CurrentLayout.depth = pScrn->depth;
-    pMga->CurrentLayout.displayWidth = pScrn->displayWidth;
-    pMga->CurrentLayout.weight.red = pScrn->weight.red;
-    pMga->CurrentLayout.weight.green = pScrn->weight.green;
-    pMga->CurrentLayout.weight.blue = pScrn->weight.blue;
-    pMga->CurrentLayout.Overlay8Plus24 = pMga->Overlay8Plus24;
-    pMga->CurrentLayout.mode = pScrn->currentMode;
-
     if (!MGAEnterVT(scrnIndex, 0))
         return FALSE;
 
@@ -3112,8 +3089,6 @@ MGACloseScreen(int scrnIndex, ScreenPtr pScreen)
     	xf86DestroyCursorInfoRec(pMga->CursorInfoRec);
     if (pMga->ShadowPtr)
 	xfree(pMga->ShadowPtr);
-    if (pMga->DGAModes)
-	xfree(pMga->DGAModes);
     if (pMga->adaptor)
 	xfree(pMga->adaptor);
     if (pMga->portPrivate)
