@@ -322,11 +322,6 @@ mgaCheckSourceTexture(int tmu, PicturePtr pPict)
     int w = pPict->pDrawable->width;
     int h = pPict->pDrawable->height;
 
-    if ((w > 2047) || (h > 2047)){
-        DEBUG_MSG(("Picture w/h too large (%dx%d)\n", w, h));
-        return FALSE;
-    }
-
     if (!mgaGetTexFormat(pPict)) {
         DEBUG_MSG(("Unsupported picture format 0x%x\n", pPict->format));
         return FALSE;
@@ -725,7 +720,7 @@ mgaDownloadFromScreen(PixmapPtr pSrc, int x, int y, int w, int h,
 {
     PMGA(pSrc);
 
-    char *src = pSrc->devPrivate.ptr;
+    CARD8 *src = pMga->FbStart + exaGetPixmapOffset(pSrc);
     int src_pitch = exaGetPixmapPitch(pSrc);
 
     int cpp = (pSrc->drawable.bitsPerPixel + 7) / 8;
@@ -734,6 +729,7 @@ mgaDownloadFromScreen(PixmapPtr pSrc, int x, int y, int w, int h,
     src += y * src_pitch + x * cpp;
 
     QUIESCE_DMA(pSrc);
+    exaWaitSync(pSrc->drawable.pScreen);
 
     while (h--) {
 	memcpy (dst, src, bytes);
@@ -780,7 +776,7 @@ init_dri(ScrnInfoPtr pScrn)
     }
 
     maxlines = (min(pMga->FbUsableSize, mb * 1024 * 1024)) /
-               (pScrn->displayWidth * pMga->CurrentLayout.bitsPerPixel / 8);
+               (pScrn->displayWidth * pScrn->bitsPerPixel / 8);
 
     dri->frontOffset = 0;
     dri->frontPitch = widthBytes;
@@ -872,6 +868,11 @@ mgaExaInit(ScreenPtr pScreen)
     pExa->maxX = 2048;
     pExa->maxY = 2048;
 
+#if EXA_VERSION_MAJOR > 2 || \
+    (EXA_VERSION_MAJOR == 2 && EXA_VERSION_MINOR > 2)
+    pExa->maxPitchPixels = 4096;
+#endif
+
     pExa->WaitMarker = mgaWaitMarker;
     pExa->PrepareSolid = mgaPrepareSolid;
     pExa->Solid = mgaSolid;
