@@ -1,3 +1,5 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_storm.c,v 1.99tsi Exp $ */
+
 #define PSZ 8
 
 #ifdef HAVE_CONFIG_H
@@ -14,16 +16,18 @@
 /* Drivers that need to access the PCI config space directly need this */
 #include "xf86Pci.h"
 
+/* Drivers for PCI hardware need this */
+#include "xf86PciInfo.h"
+
 /* Drivers that use XAA need this */
-#ifdef HAVE_XAA_H
 #include "xaa.h"
 #include "xaalocal.h"
-#endif
 #include "xf86fbman.h"
 #include "miline.h"
 #include "servermd.h"
 
-#ifdef MGADRI
+#ifdef XF86DRI
+#include "cfb.h"
 #include "GL/glxtokens.h"
 #endif
 
@@ -31,7 +35,7 @@
 #include "mga_reg.h"
 #include "mga_macros.h"
 
-#ifdef MGADRI
+#ifdef XF86DRI
 #include "mga_dri.h"
 #endif
 
@@ -61,7 +65,6 @@ do { \
   XAAMoveDWORDS((d),(s),(c)); \
 } while (0)
 
-#ifdef HAVE_XAA_H
 static void mgaSetupForSolidFill( ScrnInfoPtr pScrn, int color,
     int rop, unsigned int planemask );
 
@@ -309,6 +312,11 @@ Bool MGASetupForCPUToScreenAlphaTextureFaked( ScrnInfoPtr pScrn, int op,
 
     CHECK_DMA_QUIESCENT(pMga, pScrn);
 
+    if(pMga->Overlay8Plus24) {
+        WAITFIFO(1);
+        SET_PLANEMASK_REPLICATED( 0x00ffffff, 0xffffffff, 32 );
+    }
+
     pitch = (width + 15) & ~15;
     sizeNeeded = pitch * height;
     if(pScrn->bitsPerPixel == 16)
@@ -393,6 +401,11 @@ MGASetupForCPUToScreenAlphaTexture (
 
     CHECK_DMA_QUIESCENT(pMga, pScrn);
 
+    if(pMga->Overlay8Plus24) {
+        WAITFIFO(1);
+        SET_PLANEMASK_REPLICATED( 0x00ffffff, 0xffffffff, 32 );
+    }
+
     pitch = (width + 15) & ~15;
     sizeNeeded = (pitch * height) >> 1;
     if(pScrn->bitsPerPixel == 32)
@@ -492,6 +505,11 @@ MGASetupForCPUToScreenTexture (
 
     CHECK_DMA_QUIESCENT(pMga, pScrn);
 
+    if(pMga->Overlay8Plus24) {
+        WAITFIFO(1);
+        SET_PLANEMASK_REPLICATED( 0x00ffffff, 0xffffffff, 32 );
+    }
+
     pitch = (width + 15) & ~15;
     sizeNeeded = pitch * height;
     if(pScrn->bitsPerPixel == 16)
@@ -572,27 +590,22 @@ MGASubsequentCPUToScreenTexture (
 
 
 #endif /* defined(RENDER) */
-#endif
 
 Bool mgaAccelInit( ScreenPtr pScreen )
 {
-#ifdef HAVE_XAA_H
     XAAInfoRecPtr infoPtr;
-#endif
-    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     MGAPtr pMga = MGAPTR(pScrn);
     int maxFastBlitMem, maxlines;
     Bool doRender = FALSE;
     BoxRec AvailFBArea;
     int i;
 
-    pMga->ScratchBuffer = malloc(((pScrn->displayWidth * pMga->CurrentLayout.bitsPerPixel) + 127) >> 3);
+    pMga->ScratchBuffer = xalloc(((pScrn->displayWidth * pScrn->bitsPerPixel) + 127) >> 3);
     if(!pMga->ScratchBuffer) return FALSE;
 
-#ifdef HAVE_XAA_H
     pMga->AccelInfoRec = infoPtr = XAACreateInfoRec();
     if(!infoPtr) return FALSE;
-#endif
 
     pMga->RenderTime = 0;
     pMga->LinearScratch = 0;
@@ -609,7 +622,7 @@ Bool mgaAccelInit( ScreenPtr pScreen )
 	pMga->AccelFlags |= LARGE_ADDRESSES;
     }
 
-    if (pMga->CurrentLayout.bitsPerPixel == 24) {
+    if (pScrn->bitsPerPixel == 24) {
 	pMga->AccelFlags |= MGA_NO_PLANEMASK;
     }
 
@@ -625,7 +638,6 @@ Bool mgaAccelInit( ScreenPtr pScreen )
 	pMga->AtypeNoBLK = MGAAtypeNoBLK;
     }
 
-#ifdef HAVE_XAA_H
     /* fill out infoPtr here */
     infoPtr->Flags = 	PIXMAP_CACHE |
 			OFFSCREEN_PIXMAPS |
@@ -710,23 +722,19 @@ Bool mgaAccelInit( ScreenPtr pScreen )
     /* screen to screen color expansion */
     if(pMga->AccelFlags & USE_LINEAR_EXPANSION) {
 	infoPtr->ScreenToScreenColorExpandFillFlags =
-#if X_BYTE_ORDER == X_BIG_ENDIAN
-						BIT_ORDER_IN_BYTE_MSBFIRST;
-#else 
 						BIT_ORDER_IN_BYTE_LSBFIRST;
-#endif
 	infoPtr->SetupForScreenToScreenColorExpandFill =
 		mgaSetupForScreenToScreenColorExpandFill;
 	infoPtr->SubsequentScreenToScreenColorExpandFill =
 		mgaSubsequentScreenToScreenColorExpandFill;
     } 
-    else if ( pMga->CurrentLayout.bitsPerPixel != 24 ) {
+    else if (pScrn->bitsPerPixel != 24 ) {
 	/* Alternate (but slower) planar expansions */
 	infoPtr->SetupForScreenToScreenColorExpandFill =
 	  mgaSetupForPlanarScreenToScreenColorExpandFill;
 	infoPtr->SubsequentScreenToScreenColorExpandFill =
 	  mgaSubsequentPlanarScreenToScreenColorExpandFill;
-	infoPtr->CacheColorExpandDensity = pMga->CurrentLayout.bitsPerPixel;
+	infoPtr->CacheColorExpandDensity = pScrn->bitsPerPixel;
 	infoPtr->CacheMonoStipple = XAAGetCachePlanarMonoStipple();
 
 	/* It's faster to blit the stipples if you have fastbilt 
@@ -792,24 +800,22 @@ Bool mgaAccelInit( ScreenPtr pScreen )
     maxFastBlitMem = (pMga->Interleave ? 4096 : 2048) * 1024;
 
     if(pMga->FbMapSize > maxFastBlitMem) {
-	pMga->MaxFastBlitY = maxFastBlitMem / (pScrn->displayWidth * pMga->CurrentLayout.bitsPerPixel / 8);
+	pMga->MaxFastBlitY = maxFastBlitMem / (pScrn->displayWidth * pScrn->bitsPerPixel / 8);
     }
 
-#endif
-
     switch (pMga->Chipset) {
     case PCI_CHIP_MGAG200_SE_A_PCI:
     case PCI_CHIP_MGAG200_SE_B_PCI:
 	maxlines = (min(pMga->FbUsableSize, 1*1024*1024)) /
-		   (pScrn->displayWidth * pMga->CurrentLayout.bitsPerPixel / 8);
+		   (pScrn->displayWidth * pScrn->bitsPerPixel / 8);
 	break;
     default:
 	maxlines = (min(pMga->FbUsableSize, 16*1024*1024)) /
-		   (pScrn->displayWidth * pMga->CurrentLayout.bitsPerPixel / 8);
+		   (pScrn->displayWidth * pScrn->bitsPerPixel / 8);
 	break;
     }
 
-#ifdef MGADRI
+#ifdef XF86DRI
     if ( pMga->directRenderingEnabled ) {
        MGADRIServerPrivatePtr pMGADRIServer = pMga->DRIServerInfo;
        BoxRec MemBox;
@@ -911,7 +917,7 @@ Bool mgaAccelInit( ScreenPtr pScreen )
 		   pMGADRIServer->textureOffset );
     }
     else
-#endif /* defined(MGADRI) */
+#endif /* defined(XF86DRI) */
     {
        AvailFBArea.x1 = 0;
        AvailFBArea.x2 = pScrn->displayWidth;
@@ -932,7 +938,6 @@ Bool mgaAccelInit( ScreenPtr pScreen )
 
     }
 
-#ifdef HAVE_XAA_H
     for (i = 0; i < pScrn->numEntities; i++) {
 	if (xf86IsEntityShared(pScrn->entityList[i])) {
 	    infoPtr->RestoreAccelState = mgaRestoreAccelState;
@@ -965,18 +970,13 @@ Bool mgaAccelInit( ScreenPtr pScreen )
 #endif /* defined(RENDER) */
 
     return(XAAInit(pScreen, infoPtr));
-#else
-    return TRUE;
-#endif
 }
 
 
-#ifdef HAVE_XAA_H
 /* Support for multiscreen */
 static void mgaRestoreAccelState(ScrnInfoPtr pScrn)
 {
    MGAPtr pMga = MGAPTR(pScrn);
-   MGAFBLayout *pLayout = &pMga->CurrentLayout;
     unsigned int replicate_fg = 0;
     unsigned int replicate_bg = 0;
     unsigned int replicate_pm = 0;
@@ -985,16 +985,16 @@ static void mgaRestoreAccelState(ScrnInfoPtr pScrn)
    WAITFIFO(12);
    pMga->SrcOrg = 0;
    OUTREG(MGAREG_MACCESS, pMga->MAccess);
-   OUTREG(MGAREG_PITCH, pLayout->displayWidth);
+   OUTREG(MGAREG_PITCH, pScrn->displayWidth);
    OUTREG(MGAREG_YDSTORG, pMga->YDstOrg);
 
 
     common_replicate_colors_and_mask( pMga->FgColor, pMga->BgColor,
-				      pMga->PlaneMask, pLayout->bitsPerPixel,
+				      pMga->PlaneMask, pScrn->bitsPerPixel,
 				      & replicate_fg, & replicate_bg,
 				      & replicate_pm );
 
-    if( (pLayout->bitsPerPixel != 24)
+    if ((pScrn->bitsPerPixel != 24)
 	&& ((pMga->AccelFlags & MGA_NO_PLANEMASK) == 0) ) {
 	OUTREG( MGAREG_PLNWT, replicate_pm );
     }
@@ -1014,7 +1014,7 @@ static void mgaRestoreAccelState(ScrnInfoPtr pScrn)
    OUTREG(MGAREG_YBOT, 0x007FFFFF);    /* maxPixelPointer */
    pMga->AccelFlags &= ~CLIPPER_ON;
 }
-#endif
+
 
 CARD32 MGAAtype[16] = {
    MGADWG_RPL  | 0x00000000, MGADWG_RSTR | 0x00080000,
@@ -1071,7 +1071,6 @@ void MGAStormEngineInit( ScrnInfoPtr pScrn )
 {
     long maccess = 0;
     MGAPtr pMga = MGAPTR(pScrn);
-    MGAFBLayout *pLayout = &pMga->CurrentLayout;
     CARD32 opmode;
     static const unsigned int maccess_table[5] = {
    /* bpp:  8  16  24  32 */
@@ -1090,21 +1089,19 @@ void MGAStormEngineInit( ScrnInfoPtr pScrn )
 
     opmode = INREG(MGAREG_OPMODE);
 
-    maccess |= maccess_table[ pLayout->bitsPerPixel / 8 ];
-    if ( pLayout->depth == 15 ) {
+    maccess |= maccess_table[pScrn->bitsPerPixel / 8];
+    if (pScrn->depth == 15) {
         maccess |= (1 << 31);
     }
 
-    opmode |= opmode_table[ pLayout->bitsPerPixel / 8 ];
+    opmode |= opmode_table[pScrn->bitsPerPixel / 8];
 #if X_BYTE_ORDER == X_LITTLE_ENDIAN
     opmode &= ~0x30000;
 #endif
 
-#ifdef HAVE_XAA_H
     pMga->SetupForSolidFill = mgaSetupForSolidFill;
     pMga->SubsequentSolidFillRect = mgaSubsequentSolidFillRect;
     pMga->RestoreAccelState = mgaRestoreAccelState;
-#endif
 
 
     pMga->fifoCount = 0;
@@ -1117,7 +1114,7 @@ void MGAStormEngineInit( ScrnInfoPtr pScrn )
 						pMga->FifoSize);
     }
 
-    OUTREG(MGAREG_PITCH, pLayout->displayWidth);
+    OUTREG(MGAREG_PITCH, pScrn->displayWidth);
     OUTREG(MGAREG_YDSTORG, pMga->YDstOrg);
     OUTREG(MGAREG_MACCESS, maccess);
     pMga->MAccess = maccess;
@@ -1147,11 +1144,6 @@ void MGAStormEngineInit( ScrnInfoPtr pScrn )
     case PCI_CHIP_MGAG200_PCI:
     case PCI_CHIP_MGAG200_SE_A_PCI:
     case PCI_CHIP_MGAG200_SE_B_PCI:
-    case PCI_CHIP_MGAG200_WINBOND_PCI:
-    case PCI_CHIP_MGAG200_EW3_PCI:
-    case PCI_CHIP_MGAG200_EV_PCI:
-    case PCI_CHIP_MGAG200_EH_PCI:
-    case PCI_CHIP_MGAG200_ER_PCI:	
 	pMga->SrcOrg = 0;
 	OUTREG(MGAREG_SRCORG, pMga->realSrcOrg);
 	OUTREG(MGAREG_DSTORG, pMga->DstOrg);
@@ -1160,25 +1152,10 @@ void MGAStormEngineInit( ScrnInfoPtr pScrn )
 	break;
     }
 
-    if (pMga->is_G200WB)
-    {
-        CARD32 dwgctl = MGADWG_RSTR | 0x00060000 | MGADWG_SHIFTZERO |
-			MGADWG_BITBLT | MGADWG_BFCOL;
-        WAITFIFO(7);
-        OUTREG(MGAREG_DWGCTL, dwgctl);
-        OUTREG(MGAREG_SGN, 0);
-        OUTREG(MGAREG_AR5, 1);
-        OUTREG(MGAREG_AR0, 1);
-        OUTREG(MGAREG_AR3, 0);
-        OUTREG(MGAREG_FXBNDRY, (1 << 16) | (1 & 0xffff));
-        OUTREG(MGAREG_YDSTLEN + MGAREG_EXEC, (1 << 16) | 1);
-    }
-
     xf86SetLastScrnFlag(pScrn->entityList[0], pScrn->scrnIndex);
 }
 
 
-#ifdef HAVE_XAA_H
 static void
 MGASetClippingRectangle(
    ScrnInfoPtr pScrn,
@@ -1211,25 +1188,25 @@ MGADisableClipping(ScrnInfoPtr pScrn)
 
 
 static CARD32
-common_setup_for_pattern_fill( MGAPtr pMga, int fg, int bg, int rop,
+common_setup_for_pattern_fill(ScrnInfoPtr pScrn, int fg, int bg, int rop,
 			       int planemask,
 			       CARD32 * reg_data, unsigned int count,
 			       CARD32 cmd )
 {
+    MGAPtr pMga = MGAPTR(pScrn);
     unsigned int replicate_fg = 0;
     unsigned int replicate_bg = 0;
     unsigned int replicate_pm = 0;
     unsigned int i;
 
-
     common_replicate_colors_and_mask( fg, bg, planemask,
-				      pMga->CurrentLayout.bitsPerPixel,
+				      pScrn->bitsPerPixel,
 				      & replicate_fg, & replicate_bg,
 				      & replicate_pm );
 
 
     if( bg == -1 ) {
-    	if ( (pMga->CurrentLayout.bitsPerPixel == 24) && !RGBEQUAL(fg) ) {
+    	if ((pScrn->bitsPerPixel == 24) && !RGBEQUAL(fg) ) {
             cmd |= MGADWG_TRANSC | pMga->AtypeNoBLK[rop];
 	}
 	else {
@@ -1246,7 +1223,7 @@ common_setup_for_pattern_fill( MGAPtr pMga, int fg, int bg, int rop,
 	 */
 
 	if( ((pMga->AccelFlags & BLK_OPAQUE_EXPANSION) != 0)
-	    && ((pMga->CurrentLayout.bitsPerPixel != 24)
+	    && ((pScrn->bitsPerPixel != 24)
 		|| (RGBEQUAL(fg) && RGBEQUAL(bg))) ) {
 	    cmd |= pMga->Atype[rop];
 	}
@@ -1260,7 +1237,7 @@ common_setup_for_pattern_fill( MGAPtr pMga, int fg, int bg, int rop,
 
     SET_FOREGROUND_REPLICATED( fg, replicate_fg );
     SET_PLANEMASK_REPLICATED( planemask, replicate_pm,
-			      pMga->CurrentLayout.bitsPerPixel );
+			      pScrn->bitsPerPixel );
 
     /* FIXME: Is this the right order? */
 
@@ -1325,7 +1302,7 @@ void mgaDoSetupForScreenToScreenCopy( ScrnInfoPtr pScrn, int xdir, int ydir,
     OUTREG(MGAREG_SGN, pMga->BltScanDirection);
 
     SET_PLANEMASK_REPLICATED( planemask, replicated_mask, bpp );
-    OUTREG(MGAREG_AR5, ydir * pMga->CurrentLayout.displayWidth);
+    OUTREG(MGAREG_AR5, ydir * pScrn->displayWidth);
 }
 
 
@@ -1333,10 +1310,8 @@ void mgaSetupForScreenToScreenCopy( ScrnInfoPtr pScrn, int xdir, int ydir,
 				    int rop, unsigned int planemask,
 				    int trans )
 {
-    MGAPtr pMga = MGAPTR(pScrn);
-
     mgaDoSetupForScreenToScreenCopy( pScrn, xdir, ydir, rop, planemask, trans,
-				     pMga->CurrentLayout.bitsPerPixel );
+				     pScrn->bitsPerPixel );
 }
 
 
@@ -1349,7 +1324,7 @@ void mgaSubsequentScreenToScreenCopy( ScrnInfoPtr pScrn,
 
     if (pMga->AccelFlags & LARGE_ADDRESSES) {
 	const unsigned int display_bit_width =
-	  (pMga->CurrentLayout.displayWidth * pMga->CurrentLayout.bitsPerPixel);
+	  (pScrn->displayWidth * pScrn->bitsPerPixel);
 
 	SrcOrg = ((srcY & ~1023) * display_bit_width) >> 9;
 	DstOrg = ((dstY & ~1023) * display_bit_width) >> 9;
@@ -1376,7 +1351,7 @@ void mgaSubsequentScreenToScreenCopy( ScrnInfoPtr pScrn,
 	    OUTREG(MGAREG_SRCORG, (SrcOrg << 6) + pMga->realSrcOrg);
  	}
 	if(SrcOrg) {
-	    SrcOrg = (SrcOrg << 9) / pMga->CurrentLayout.bitsPerPixel;
+	    SrcOrg = (SrcOrg << 9) / pScrn->bitsPerPixel;
 	    end -= SrcOrg;
 	    start -= SrcOrg;
 	}
@@ -1418,7 +1393,7 @@ void mgaSubsequentScreenToScreenCopy_FastBlit( ScrnInfoPtr pScrn,
 
     /* we assume the driver asserts screen pitches such that
 	we can always use fastblit for scrolling */
-    if(((srcX ^ dstX) & masks[ pMga->CurrentLayout.bitsPerPixel / 8 ]) == 0) {
+    if(((srcX ^ dstX) & masks[pScrn->bitsPerPixel / 8]) == 0) {
 	if(pMga->MaxFastBlitY) {
 	   if(pMga->BltScanDirection & BLIT_UP) {
 		if((srcY >= pMga->MaxFastBlitY) ||
@@ -1439,9 +1414,9 @@ void mgaSubsequentScreenToScreenCopy_FastBlit( ScrnInfoPtr pScrn,
 	    static const unsigned shift_tab[5] = {
 		0, 6, 5, 6, 4
 	    };
-	    const unsigned shift = shift_tab[pMga->CurrentLayout.bitsPerPixel / 8];
+	    const unsigned shift = shift_tab[pScrn->bitsPerPixel / 8];
 	    
-	   if (pMga->CurrentLayout.bitsPerPixel == 24) {
+	   if (pScrn->bitsPerPixel == 24) {
 	       tmp_dstX *= 3;
 	       tmp_fxright = fxright * 3 + 2;
 	   }
@@ -1449,7 +1424,7 @@ void mgaSubsequentScreenToScreenCopy_FastBlit( ScrnInfoPtr pScrn,
            if( (tmp_dstX & (1 << shift)) 
 	       && (((tmp_fxright >> shift) - (tmp_dstX >> shift)) & 7) == 7) {
 	       fxright = (tmp_fxright | (1 << shift));
-	       if (pMga->CurrentLayout.bitsPerPixel == 24) {
+	       if (pScrn->bitsPerPixel == 24) {
 		   fxright /= 3;
 	       }
 
@@ -1541,10 +1516,8 @@ void mgaDoSetupForSolidFill( ScrnInfoPtr pScrn, int color, int rop,
 void mgaSetupForSolidFill( ScrnInfoPtr pScrn, int color, int rop,
 			   unsigned int planemask )
 {
-    MGAPtr pMga = MGAPTR(pScrn);
-
     mgaDoSetupForSolidFill( pScrn, color, rop, planemask, 
-			    pMga->CurrentLayout.bitsPerPixel );
+			    pScrn->bitsPerPixel);
 }
 
 void mgaSubsequentSolidFillRect( ScrnInfoPtr pScrn, 
@@ -1647,7 +1620,7 @@ void mgaSetupForMono8x8PatternFill( ScrnInfoPtr pScrn,
     regs[2] = MGAREG_PAT1;
     regs[3] = paty;
 
-    pMga->PatternRectCMD = common_setup_for_pattern_fill( pMga, fg, bg, rop,
+    pMga->PatternRectCMD = common_setup_for_pattern_fill(pScrn, fg, bg, rop,
 							  planemask, regs, 2,
 							  (MGADWG_TRAP
 							   | MGADWG_ARZERO
@@ -1727,7 +1700,7 @@ void mgaSetupForScanlineCPUToScreenColorExpandFill( ScrnInfoPtr pScrn,
 
     CHECK_DMA_QUIESCENT(pMga, pScrn);
 
-    (void) common_setup_for_pattern_fill( pMga, fg, bg, rop,
+    (void) common_setup_for_pattern_fill( pScrn, fg, bg, rop,
 					  planemask, NULL, 0,
 					  MGADWG_ILOAD | MGADWG_LINEAR 
 					  | MGADWG_SGNZERO | MGADWG_SHIFTZERO
@@ -1853,7 +1826,7 @@ void mgaSetupForScanlineImageWrite( ScrnInfoPtr pScrn, int rop,
     MGAPtr pMga = MGAPTR(pScrn);
     unsigned int replicate_pm = 0;
 
-    switch( pMga->CurrentLayout.bitsPerPixel ) {
+    switch (pScrn->bitsPerPixel) {
     case 8:
 	replicate_pm = REPLICATE_8( planemask );
 	break;
@@ -1873,7 +1846,7 @@ void mgaSetupForScanlineImageWrite( ScrnInfoPtr pScrn, int rop,
     WAITFIFO(3);
     OUTREG(MGAREG_AR5, 0);
     SET_PLANEMASK_REPLICATED( planemask, replicate_pm,
-			      pMga->CurrentLayout.bitsPerPixel );
+			      pScrn->bitsPerPixel );
     OUTREG(MGAREG_DWGCTL, MGADWG_ILOAD | MGADWG_BFCOL | MGADWG_SHIFTZERO |
 			MGADWG_SGNZERO | pMga->AtypeNoBLK[rop]);
 }
@@ -1887,7 +1860,7 @@ void mgaSubsequentScanlineImageWriteRect( ScrnInfoPtr pScrn,
 
     pMga->AccelFlags |= CLIPPER_ON;
     pMga->expandRows = h;
-    pMga->expandDWORDs = ((w * pMga->CurrentLayout.bitsPerPixel) + 31) >> 5;
+    pMga->expandDWORDs = ((w * pScrn->bitsPerPixel) + 31) >> 5;
 
     WAITFIFO(5);
     OUTREG(MGAREG_CXBNDRY, 0xFFFF0000 | ((x + skipleft) & 0xFFFF));
@@ -1941,7 +1914,7 @@ void mgaSetupForDashedLine( ScrnInfoPtr pScrn,
 
 
     common_replicate_colors_and_mask( fg, bg, planemask,
-				      pMga->CurrentLayout.bitsPerPixel,
+				      pScrn->bitsPerPixel,
 				      & replicate_fg, & replicate_bg,
 				      & replicate_pm );
 
@@ -1959,7 +1932,7 @@ void mgaSetupForDashedLine( ScrnInfoPtr pScrn,
     }
 
     SET_PLANEMASK_REPLICATED( planemask, replicate_pm,
-			      pMga->CurrentLayout.bitsPerPixel );
+			      pScrn->bitsPerPixel );
     SET_FOREGROUND_REPLICATED( fg, replicate_fg );
 
 
@@ -1979,7 +1952,7 @@ void mgaSetupForDashedLine( ScrnInfoPtr pScrn,
 	pMga->AccelFlags |= NICE_DASH_PATTERN;
 
 	if( bg == -1 ) {
-	    if ( (pMga->CurrentLayout.bitsPerPixel == 24) && !RGBEQUAL(fg) ) {
+	    if ((pScrn->bitsPerPixel == 24) && !RGBEQUAL(fg)) {
 		pMga->NiceDashCMD |= MGADWG_TRANSC | pMga->AtypeNoBLK[rop];
 	    }
 	    else {
@@ -1994,7 +1967,7 @@ void mgaSetupForDashedLine( ScrnInfoPtr pScrn,
 	     */
 
 	    if( ((pMga->AccelFlags & BLK_OPAQUE_EXPANSION) != 0)
-		&& ((pMga->CurrentLayout.bitsPerPixel != 24)
+		&& ((pScrn->bitsPerPixel != 24)
 		    || (RGBEQUAL(fg) && RGBEQUAL(bg))) ) {
 		pMga->NiceDashCMD |= pMga->Atype[rop];
 	    }
@@ -2071,7 +2044,7 @@ void mgaSetupForPlanarScreenToScreenColorExpandFill( ScrnInfoPtr pScrn,
 
     CHECK_DMA_QUIESCENT(pMga, pScrn);
 
-    (void) common_setup_for_pattern_fill( pMga, fg, bg, 0, planemask, regs, 1,
+    (void) common_setup_for_pattern_fill( pScrn, fg, bg, 0, planemask, regs, 1,
 					  mgaCMD );
 }
 
@@ -2109,11 +2082,11 @@ void mgaSetupForScreenToScreenColorExpandFill( ScrnInfoPtr pScrn,
     CARD32 regs[2];
    
     regs[0] = MGAREG_AR5;
-    regs[1] = pScrn->displayWidth * pMga->CurrentLayout.bitsPerPixel;
+    regs[1] = pScrn->displayWidth * pScrn->bitsPerPixel;
 
     CHECK_DMA_QUIESCENT(pMga, pScrn);
 
-    (void) common_setup_for_pattern_fill( pMga, fg, bg, rop, planemask,
+    (void) common_setup_for_pattern_fill( pScrn, fg, bg, rop, planemask,
 					  regs, 1,
 					  MGADWG_BITBLT | MGADWG_SGNZERO 
 					  | MGADWG_SHIFTZERO );
@@ -2127,7 +2100,7 @@ void mgaSubsequentScreenToScreenColorExpandFill( ScrnInfoPtr pScrn,
 {
     MGAPtr pMga = MGAPTR(pScrn);
     const unsigned int display_bit_width =
-      (pMga->CurrentLayout.displayWidth * pMga->CurrentLayout.bitsPerPixel);
+      (pScrn->displayWidth * pScrn->bitsPerPixel);
     int start, end, next, num;
     Bool resetDstOrg = FALSE;
 
@@ -2150,7 +2123,7 @@ void mgaSubsequentScreenToScreenColorExpandFill( ScrnInfoPtr pScrn,
     }
 
     w--;
-    start = (XYADDRESS(srcx, srcy) * pMga->CurrentLayout.bitsPerPixel)
+    start = (XYADDRESS(srcx, srcy) * pScrn->bitsPerPixel)
       + skipleft;
     end = start + w + (display_bit_width * (h - 1));
 
@@ -2361,7 +2334,7 @@ MGAValidatePolyArc(
    unsigned long changes,
    DrawablePtr pDraw
 ){
-   ScrnInfoPtr pScrn = xf86ScreenToScrn(pGC->pScreen);
+   ScrnInfoPtr pScrn = xf86Screens[pGC->pScreen->myNum];
    MGAPtr pMga = MGAPTR(pScrn);
    Bool fullPlanemask = TRUE;
 
@@ -2448,7 +2421,7 @@ MGAValidatePolyPoint(
    unsigned long changes,
    DrawablePtr pDraw
 ){
-   ScrnInfoPtr pScrn = xf86ScreenToScrn(pGC->pScreen);
+   ScrnInfoPtr pScrn = xf86Screens[pGC->pScreen->myNum];
    MGAPtr pMga = MGAPTR(pScrn);
    Bool fullPlanemask = TRUE;
 
@@ -2558,4 +2531,3 @@ MGAFillCacheBltRects(
 
     SET_SYNC_FLAG(infoRec);
 }
-#endif
